/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

/**
 * @ingroup Database client library
 * @file
 * Interface for communication with the trackframe database server.
 *
 * @author Peter Keitler <keitler@in.tum.de>
 */ 

#ifndef __DBCLIENT_H__
#define __DBCLIENT_H__

#ifdef _WIN32
#	ifdef UBITRACK_DLL
#		define DBCLIENT_EXPORT __declspec( dllexport )
#	else
#		define DBCLIENT_EXPORT __declspec( dllimport )
#	endif
#else // _WIN32
#	define DBCLIENT_EXPORT
#endif

#include <ios>
#include <istream>
#include <iostream>


namespace DbClient {


/**
 * Initialize the trackframe database client library. 
 * This function must be called before all other functions.
 *
 * @return true if successful
 */
DBCLIENT_EXPORT bool init( void );


/**
 * Retrieve input stream for receiving data from trackframe database server.
 * Call closeStream() when you do not need the input stream any longer.
 *
 * @note Currently, the library only supports \em one input stream to be opened.
 *
 * @param URL specifying which information to fetch from database server
 * @return Pointer to input stream in case of success, NULL otherwise
 */
DBCLIENT_EXPORT std::istream * getInputStream( char * url );


/**
 * Close a stream, e.g. one retrieved via getInputStream(), after data has
 * been fully received and the stream is not needed any longer.
 *
 * @param Pointer to stream
 * @return true if successful
 */
DBCLIENT_EXPORT bool closeStream( std::ios * stream );


/**
 * Allow database client library to do its cleanup when all work is done.
 * @return true if successful
 */
DBCLIENT_EXPORT bool cleanup( void );


} // namespace DbClient



#endif
