/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */


#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <boost/thread.hpp>

#include <Ubitrack/Interface.h>
#include <Ubitrack/Util/Exception.h>

#include <log4cpp/PropertyConfigurator.hh>

#include <DatabaseClient/DatabaseClient.h>

using namespace DbClient;
using namespace Ubitrack;
using namespace std;

int main( int ac, char** av )
{
    try	
    {
	if (ac < 2) {
	    std::cerr << "Usage: " << av[0] << " filename" << std::endl;
	    exit (1);
	}

	// try to initialize logging from file
	try
	{ log4cpp::PropertyConfigurator::configure( "log4cpp.conf" ); }
	catch ( ... )
	{ } // no nothing

	std::cout << "Loading components..." << std::endl << std::flush;
	if ( !initDataflow( getenv( "UBITRACK_COMPONENTS_PATH" ) ) )
	    UBITRACK_THROW( "Problem in initDataflow()" );

	std::cout << "Instantiating network from " << av[1] << "..." << std::endl << std::flush;

	// Initialize database client library
	DbClient::init();
	// Get input stream from URL
	istream * is = DbClient::getInputStream( av[1] );
	if ( is == NULL )
	    UBITRACK_THROW( "Problem in dbclient_get_srg_input_stream()" );

	// Pass SRG stream to Ubitrack
	if ( !loadDataflow( is ) )
	    UBITRACK_THROW( "Problem in loadDataflow()" );

	// Close stream
	DbClient::closeStream( is );
	// Be polite, cleanup library
	DbClient::cleanup();

	std::cout << "Starting dataflow" << std::endl;
	startDataflow();

	std::cout << "Press RETURN to stop..." << std::endl << std::flush;
	std::cin.get();

	stopDataflow();
	std::cout << "Finished, cleaning up..." << std::endl << std::flush;
		
	cleanupDataflow();
    }
    catch( Util::Exception& e )	
    {
	std::cerr << e << std::endl;
    }
}

