/* Arbitrary module name */

%module (directors="1") ubitrack


/* Includes that will be added to the generated
xxx_wrap.c/xxx_wrap.cpp wrapper file. They will not be interpreted by
SWIG */

%{
#include <string>
#include "Ubitrack/config.h"
#include "Ubitrack/Facade/SimpleFacade.h"
#include "Ubitrack/Util/Logging.h"
%}



/* Generate directors for C++ classes for which cross language polymorphism
shall be provided. This is especially needed for enabling C++ code to callback
Java methods.  This must come before the C/C++ declarations to be wrapped. */

%feature("director") SimplePoseReceiver;
%feature("director") SimpleDistanceReceiver;
%feature("director") SimpleButtonReceiver;
%feature("director") SimpleStringReceiver;
%feature("director") SimpleImageReceiver;
%feature("director") SimpleDataflowObserver;


/* Needed to transfer memory ownership to Java, e.g. for C++ factory methods 
which create and return objects that are not known/owned/used in C++ code */

%newobject Ubitrack::Facade::SimpleFacade::getSimplePullSinkPose( const char * );


/* Needed for array access to be possible from Java. */

#ifdef SWIGJAVA
%include "carrays.i";
%array_functions(signed char, byteArray);
#endif


/* needed for stl strings to be accessible as target language strings */
%include "std_string.i"


/* C/C++ declarations to be wrapped.  The paths are relative to the
working directory of swig, which is the directory where the invoking
SConscript file resides.  Attention: The order of includes matters!
Declarations must come before their usage, otherwise, SWIG will
generate generic type wrapper classes of style SWITTYPE_p_<xxx>. */

%include ../../Ubitrack/config.h
%include ../../Ubitrack/Ubitrack.h
%include ../../Ubitrack/Facade/SimpleDatatypes.h
%include ../../Ubitrack/Facade/SimpleFacade.h
%include ../../Ubitrack/Util/Logging.h


/* also wrap the dataflow generation (pattern matching) */
%inline %{
namespace Ubitrack { namespace Graph {
	std::string generateDataflow( const std::string& query );
	std::string dataflowToGraphviz( const std::string& dataflow );
}}
%}
