/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

#ifndef __RECONFIGURATOR_CONVEXHULL_H_INCLUDED__
#define __RECONFIGURATOR_CONVEXHULL_H_INCLUDED__

#include <vector>
#include <tinyxml.h>
#include <Ubitrack/Math/Vector.h>


/** 
 * represents the convex hull of a point set.
 * Warning: O( n^4 ) complexity!
 */
class ConvexHull
{
public:
	/** construct from xml ConvexHull node */
	ConvexHull( const TiXmlNode* pXml );

	/** positive = outside, negative = inside */
	double distance( const Ubitrack::Math::Vector< 3 >& point ) const;
	
protected:
	/** represents a face on the convex hull */
	class Plane
	{
	public:
		/** construct from three points */
		Plane( const Ubitrack::Math::Vector< 3 >& p1, const Ubitrack::Math::Vector< 3 >& p2, const Ubitrack::Math::Vector< 3 >& p3 );
		
		/** compute the distance of a point from the plane */
		double distance( const Ubitrack::Math::Vector< 3 >& p ) const
		{ return boost::numeric::ublas::inner_prod( p - m_point, m_normal ); }
		
		/** inverts the direction of the normal vector */
		void swapNormal()
		{ m_normal *= -1; }
		
	protected:
		Ubitrack::Math::Vector< 3 > m_point;
		Ubitrack::Math::Vector< 3 > m_normal;
	};
	
	std::vector< Plane > m_planes;
};

#endif
