/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

#ifndef __RECONFIGURATOR_RECONFIGURATIONREGION_H_INCLUDED__
#define __RECONFIGURATOR_RECONFIGURATIONREGION_H_INCLUDED__

#include <string>
#include <boost/shared_ptr.hpp>
#include <tinyxml.h>

#include <Ubitrack/Facade/AdvancedFacade.h>
#include <Ubitrack/Facade/DataflowObserver.h>
#include <Ubitrack/Measurement/Measurement.h>

#include "ConvexHull.h"


/** a region that does something when an object enters or leaves */
class ReconfigurationRegion
	: protected Ubitrack::Facade::DataflowObserver
{
public:
	/** construct from TiXml node */
	ReconfigurationRegion( const TiXmlElement* pXml, Ubitrack::Facade::AdvancedFacade& facade );

	/** virtual destructor, just to have one... */
	virtual ~ReconfigurationRegion()
	{}

	/** must be called regularly */
	void check(); 
	
	/** returns the UTQL pattern (query) for the receiver */
	const std::string& getReceiverPattern() const
	{ return m_sReceiverPattern; }
	
protected:
	/** possible interal states */
	enum RegionState { inside, outside, undecided };

	/** changes the internal state */
	void changeState( RegionState newState );

	/** receives the actual tracking input */
	void receiveTracking( const Ubitrack::Measurement::Position& m );

	/** Part of DataflowObserver interface, called when a data flow component is added */
	virtual void notifyAddComponent( const std::string& sPatternName, const std::string& sPatternId, const Ubitrack::Graph::UTQLSubgraph& );

	/** Part of DataflowObserver interface, called when a data flow component is removed */
	virtual void notifyDeleteComponent( const std::string& sPatternName, const std::string& sPatternId );

	/** the state */
	RegionState m_state;
	
	/** pointer to convex hull, may be empty */
	boost::shared_ptr< ConvexHull > m_pConvexHull;
	
	/** timeout in ms */
	int m_timeout;
	
	/** hysteresis in m */
	double m_hysteresis;
	
	/** time of last event */
	Ubitrack::Measurement::Timestamp m_lastChangeTime;
	
	std::string m_sEnterAction;
	std::string m_sLeaveAction;
	std::string m_sReceiverName;
	std::string m_sReceiverPattern;
	
	/** reference to facade */
	Ubitrack::Facade::AdvancedFacade& m_rFacade;
};

#endif
