/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

/**
 * @file
 * Header file of a TCP socket acceptor for the ubitrack server
 *
 * @author Daniel Pustka <daniel.pustka@in.tum.de>
 */ 

#ifndef __UBITRACKSERVER_TCPACCEPTOR_H_INCLUDED__
#define __UBITRACKSERVER_TCPACCEPTOR_H_INCLUDED__

#include <list>

#include <boost/shared_ptr.hpp>
#include <boost/scoped_ptr.hpp>
#include <asio.hpp>
#include <boost/thread.hpp>

#include <Ubitrack/ClientServer/TcpConnection.h>

// forward decls
class UtqlServer;


/**
 * Accepts tcp socket connections and adds the clients to the ubitrack server
 */
class TcpAcceptor
{
public:
	/**
	 * Constructor
	 * @param nPort port number to listen to
	 * @param server reference to the ubitrack UTQL server object
	 */
	TcpAcceptor( int nPort, UtqlServer& server );
	
	/** destructor */
	~TcpAcceptor();

	/** remove dead clients */
	void checkDeadClients();
	
protected:
	/** Asio callback for connects */
	void handleAccept( const asio::error_code );

	/** the asio io_service for dispatching events */
	asio::io_service m_ioService;
	
	/** more asio stuff */
	asio::ip::tcp::acceptor m_acceptor;
	
	/** a spare socket to accept the next connection */
	boost::shared_ptr< asio::ip::tcp::socket > m_pSpareSocket;
	
	/** the thread for dispatching events */
	boost::scoped_ptr< boost::thread > m_pThread;

	/** shortcut for list of connections type */
	typedef std::list< boost::shared_ptr< Ubitrack::ClientServer::TcpConnection > > ConnectionList;
	
	/** list of all connections */
	ConnectionList m_connections;
	
	/** reference to the server */
	UtqlServer& m_rServer;
};

#endif
