/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

/**
 * @file
 * Header of the UtqlServer
 *
 * @author Daniel Pustka <daniel.pustka@in.tum.de>
 */

#ifndef __UBITRACKSERVER_UTQLSERVER_H_INCLUDED__
#define __UBITRACKSERVER_UTQLSERVER_H_INCLUDED__

#include <list>
#include <Ubitrack/ClientServer/ClientServerConnection.h>
#include <Ubitrack/Graph/UTQLServer.h>

/**
 * A class for a ubitrack server that talks to clients using the
 * ClientServerConnection interface.
 *
 * So far, this is just a skeleton where real functionality will be added later.
 */
class UtqlServer
{
public:
	/** shortcut for client connection pointer type */
	typedef boost::shared_ptr< Ubitrack::ClientServer::ClientServerConnection > ClientPtr;

	/**
	 * Constructor
	 */
	UtqlServer();

	/**
	 * Add a new client to the server
	 * @param pConnection shared_ptr to the client interface
	 */
	void addClient( ClientPtr pConnection );

	/**
	 * Remove a client from the server
	 * @param pConnection shared_ptr to the client interface
	 */
	void removeClient( ClientPtr pConnection );

protected:
	/** Contains internal information the server needs to maintain for each client */
	struct ClientInfo
	{
		/** pointer to the client interface */
		ClientPtr pClient;

		// whatever else we need...
	};

	/** the list of all clients */
	std::list< ClientInfo > m_clients;

	/** shortcut for client iterator type */
	typedef std::list< ClientInfo >::iterator ClientIterator;

	/**
	 * The method that receives incoming UTQL queries from clients
	 * @param itClient iterator to the \c ClientInfo in the client list
	 * @param pData pointer to the received data
	 */
	void handleQuery( ClientIterator itClient, boost::shared_ptr< Ubitrack::ClientServer::ClientServerConnection::BufferType > pData );

	Ubitrack::Graph::UTQLServer m_Server;
};

#endif
