/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */


/**
 * @ingroup dataflow_components
 * @file
 * Average component.
 * This class calculates the average of a PositionList(2D) measurement.a
 *
 * @author Florian Echtler <echtler@in.tum.de>
 */

#include <boost/bind.hpp>
//#include <log4cpp/Category.hh>

#include <Ubitrack/Dataflow/PullConsumer.h>
#include <Ubitrack/Dataflow/PushConsumer.h>
#include <Ubitrack/Dataflow/PushSupplier.h>
#include <Ubitrack/Dataflow/Component.h>
#include <Ubitrack/Dataflow/ComponentFactory.h>
#include <Ubitrack/Measurement/Measurement.h>

// currently not used
//static log4cpp::Category& logger( log4cpp::Category::getInstance( "Ubitrack.Components.Average" ) );

namespace Ubitrack { namespace Components {

using namespace Dataflow;

template< class EventType > class Average : public Component
{
public:
	/**
	 * Standard component constructor.
	 *
	 * @param nm Unique name of the component.
	 * @param cfg ComponentConfiguration containing all configuration.
	 */
	Average( const std::string& nm, boost::shared_ptr< Graph::UTQLSubgraph > pCfg )
		: Ubitrack::Dataflow::Component( nm )
		, m_inPort( "Input", *this, boost::bind( &Average::receive, this, _1 ) )
		, m_outPort( "Output", *this )
	{
	}

protected:

	typedef Measurement::Measurement< typename std::vector< typename EventType::value_type > > EventList;

	/** called when a new item arrives */
	void receive( const EventList& event )
	{
		typename EventType::value_type tmp;
		int size = event->size();
		for (int i = 0; i < size; i++) tmp += ((*event)[i]/size);
		m_outPort.send( EventType( event.time(), tmp ) );
	}

	/** Ports of the component */
	PushConsumer< EventList > m_inPort;
	PushSupplier< EventType > m_outPort;

};

UBITRACK_REGISTER_COMPONENT( ComponentFactory* const cf ) 
{
	cf->registerComponent< Average< Measurement::Position > > ( "PositionListAverage" );
	cf->registerComponent< Average< Measurement::Position2D > > ( "PositionList2DAverage" );
}

} } // namespace Ubitrack::Components

