/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

/**
 * @file
 * Serial port class.
 *
 * @author Manuel Huber <huberma@in.tum.de>
 */ 


#ifndef __UBITRACK_UTIL_SERIALPORT_H_INCLUDED__
#define __UBITRACK_UTIL_SERIALPORT_H_INCLUDED__



#include <string>
#include <Ubitrack/Ubitrack.h>

#ifdef _WIN32

	#include <Ubitrack/Util/CleanWindows.h>

#else // *nix

	#ifndef __APPLE__
		#include <termio.h>
	#else
		#include <sys/ioctl.h>
		#include <termios.h>
	#endif

#endif

namespace Ubitrack { namespace Util {

class UBITRACK_EXPORT SerialPort
{
public:
	SerialPort( std::string port, unsigned long baudRate )
		: m_portName( port )
		, m_baudRate( baudRate )
		, m_portOpen( false )
	{}

	~SerialPort( )
	{
		close();
	}


	void open( int vtime = 5, int vmin = 0 );
	void close();

	unsigned long send( const unsigned char* buffer, unsigned long size );
	unsigned long read( unsigned char* buffer, unsigned long size );

	unsigned long bytesOnRead();
	void flush();

	unsigned long getBaudRate()
	{
		return m_baudRate;
	}

protected:

	std::string m_portName;
	unsigned long m_baudRate;
	bool m_portOpen;

#ifdef _WIN32

	HANDLE        m_hSerialPort;

#else // *nix

	int m_fileDescriptor;
	struct termios m_termiosCurrent;
	struct termios m_termiosOriginal;

#endif

};

} } // namespace Ubitrack::Util

#endif


