/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

#include "EasyLaseModule.h"
#include <log4cpp/Category.hh>
#include <Ubitrack/Util/OS.h>

using namespace Ubitrack::Dataflow;

static log4cpp::Category& logger( log4cpp::Category::getInstance( "Drivers.EasyLase" ) );

namespace Ubitrack { namespace Drivers {


void LaserProjector::redraw( )
{
	// create a transformed list of objects
	m_frame.clear();
	ComponentList objects = getAllComponents();
	for ( unsigned int i = 0; i < objects.size(); i++ ) {
		std::vector<LaserPoint>* vec = objects[i]->getPattern();
		m_frame.insert( m_frame.end(), vec->begin(), vec->end() );
	}

	// wait until the device is not busy, then send the frame
	if ( easyLaseGetStatus( m_num ) == EASYLASE_GET_STATUS_BUSY ) 
	{
		LOG4CPP_INFO( logger, "EasyLase busy. Dropping event." );
		return;
	}

/*	while ( easyLaseGetStatus( m_num ) == EASYLASE_GET_STATUS_BUSY ) 
	{ 
		Util::sleep( 10 ); 
	} */
	int res = easyLaseWriteFrame( m_num, (unsigned char*)&(m_frame[0]), m_frame.size()*sizeof(LaserPoint), m_pps );
	#ifdef WIN32
	if (!res) 
	#else
	if (res)
	#endif
		LOG4CPP_INFO( logger, "USB Error reported by easyLaseWriteFrame (probably too fast)." );
}


LaserProjector::LaserProjector( const LaserProjectorKey& key, boost::shared_ptr< Graph::UTQLSubgraph >, FactoryHelper* pFactory )
	: Module< LaserProjectorKey, LaserObjectKey, LaserProjector, LaserObject >( key, pFactory )
	, m_num(key.m_num)
	, m_pps(key.m_pps)
	, m_frame()
{
	int devcount = easyLaseGetCardNum();
	if (devcount < m_num+1) UBITRACK_THROW( "Requested EasyLase USB device not found." );
}


LaserProjector::~LaserProjector()
{
	easyLaseClose();
}


/** register module at factory */
UBITRACK_REGISTER_COMPONENT( ComponentFactory* const cf )
{
	cf->registerModule< LaserProjector > ( "EasyLaseModule" );
}

} } // namespace Ubitrack::Drivers
