/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */


/**
 * @ingroup driver_components
 * @file
 * GPS driver
 * This file contains the driver component to
 * get NMEA location data from a GPS receiver.
 *
 * The received data is sent via a push interface.
 *
 * @author Florian Echtler <echtler@in.tum.de>
 */

#include <map>
#include <string>
#include <cstdlib>
#include <log4cpp/Category.hh>

#include <asio.hpp>

#include <Ubitrack/Dataflow/PushSupplier.h>
#include <Ubitrack/Dataflow/ComponentFactory.h>
#include <Ubitrack/Dataflow/Component.h>
#include <Ubitrack/Measurement/Measurement.h>
#include <Ubitrack/Measurement/TimestampSync.h>
#include <Ubitrack/Util/SerialPort.h>
#include <Ubitrack/Util/Exception.h>
#include <Ubitrack/Util/OS.h>


#include <Ubitrack/Measurement/TimestampSync.h>

namespace Ubitrack { namespace Drivers {

class MausometerDriver
	: public Dataflow::Component
{

public:

	/** constructor */
	MausometerDriver( const std::string& sName, boost::shared_ptr< Graph::UTQLSubgraph > pConfig );

	/** destructor */
	~MausometerDriver();

protected:

	/** thread method */
	void HandleReceive(const asio::error_code, size_t length);

	void processLine(Ubitrack::Measurement::Timestamp time);

	boost::shared_ptr< Dataflow::PushSupplier< Measurement::Position2D > > m_sensorPorts[3];
	boost::shared_ptr< Dataflow::PushSupplier< Measurement::Button > > m_buttonPorts[3];

	boost::shared_ptr< boost::thread > m_thread;
	int m_keepRunning;

    enum { max_receive_length = 10240 , receive_buffer_size = 10242 };

    boost::shared_ptr< asio::io_service > m_IoService;
    boost::shared_ptr< asio::ip::tcp::socket > m_Socket;

    // Recive data. Do not touch from outside of async network thread

    char receive_data[receive_buffer_size];
	char receive_line[receive_buffer_size];
	char* receive_line_ptr;
	char* receive_line_end_ptr;

	Measurement::TimestampSync m_synchronizer;

	asio::ip::tcp::endpoint mouse_endpoint;

};

} } // namespace Ubitrack::Drivers
