/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */


#include "Network2Sink.h"
#include "Network2Source.h"
#include <Ubitrack/Measurement/Measurement.h>

namespace Ubitrack { namespace Drivers {

boost::shared_ptr< Network2ComponentBase > Network2Module::createComponent( const std::string& type, const std::string& name, 
	boost::shared_ptr< Graph::UTQLSubgraph > config, const Network2Module::ComponentKey& key, Network2Module* pModule )
{
	if ( type == "Network2SinkPose" )
		return boost::shared_ptr< Network2ComponentBase >( new Network2Sink< Measurement::Pose >( name, config, key, pModule ) );
	else if ( type == "Network2SinkErrorPose" )
		return boost::shared_ptr< Network2ComponentBase >( new Network2Sink< Measurement::ErrorPose >( name, config, key, pModule ) );
	else if ( type == "Network2SinkRotation" )
		return boost::shared_ptr< Network2ComponentBase >( new Network2Sink< Measurement::Rotation >( name, config, key, pModule ) );
	else if ( type == "Network2SinkPosition" )
		return boost::shared_ptr< Network2ComponentBase >( new Network2Sink< Measurement::Position >( name, config, key, pModule ) );
	else if ( type == "Network2SinkPositionList" )
		return boost::shared_ptr< Network2ComponentBase >( new Network2Sink< Measurement::PositionList >( name, config, key, pModule ) );
	else if ( type == "Network2SinkPositionList2" )
		return boost::shared_ptr< Network2ComponentBase >( new Network2Sink< Measurement::PositionList2 >( name, config, key, pModule ) );
		
	else if ( type == "Network2SourcePose" )
		return boost::shared_ptr< Network2ComponentBase >( new Network2Source< Measurement::Pose >( name, config, key, pModule ) );
	else if ( type == "Network2SourceErrorPose" )
		return boost::shared_ptr< Network2ComponentBase >( new Network2Source< Measurement::ErrorPose >( name, config, key, pModule ) );
	else if ( type == "Network2SourceRotation" )
		return boost::shared_ptr< Network2ComponentBase >( new Network2Source< Measurement::Rotation >( name, config, key, pModule ) );
	else if ( type == "Network2SourcePosition" )
		return boost::shared_ptr< Network2ComponentBase >( new Network2Source< Measurement::Position >( name, config, key, pModule ) );
	else if ( type == "Network2SourcePositionList" )
		return boost::shared_ptr< Network2ComponentBase >( new Network2Source< Measurement::PositionList >( name, config, key, pModule ) );
	else if ( type == "Network2SourcePositionList2" )
		return boost::shared_ptr< Network2ComponentBase >( new Network2Source< Measurement::PositionList2 >( name, config, key, pModule ) );
		
	UBITRACK_THROW( "Class " + type + " not supported by network source module." );
}

} } // namespace Ubitrack::Drivers


// register module at factory
UBITRACK_REGISTER_COMPONENT( Ubitrack::Dataflow::ComponentFactory* const cf ) {

	// create list of supported types
	std::vector< std::string > network2Components;

	network2Components.push_back( "Network2SinkPose" );
	network2Components.push_back( "Network2SinkErrorPose" );
	network2Components.push_back( "Network2SinkRotation" );
	network2Components.push_back( "Network2SinkPosition" );
	network2Components.push_back( "Network2SinkPositionList" );
	network2Components.push_back( "Network2SinkPositionList2" );

	network2Components.push_back( "Network2SourcePose" );
	network2Components.push_back( "Network2SourceErrorPose" );
	network2Components.push_back( "Network2SourceRotation" );
	network2Components.push_back( "Network2SourcePosition" );
	network2Components.push_back( "Network2SourcePositionList" );
	network2Components.push_back( "Network2SourcePositionList2" );

	cf->registerModule< Ubitrack::Drivers::Network2Module >( network2Components );
}


