/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */


/**
 * @ingroup driver_components
 * @file
 * RequestPacket.h
 * 
 * Defines the packet contents for requesting tracking data from a source.
 *
 * @author Daniel Pustka <pustka@in.tum.de>
 */

#ifndef _NETWORK2_REQUESTPACKET_H_
#define _NETWORK2_REQUESTPACKET_H_

#include <string>

namespace Ubitrack { namespace Drivers {

struct RequestPacket
{
	/** host name to send tracking data to (if empty, sender IP of request will be taken from UDP packet) */
	std::string sHost;

	/** port to send tracking data to (if 0, sender port of request will be taken from UDP packet) */
	int nPort;

	/** receiver component name */
	std::string sComponentName;

	/** boost::archive serialization method */
	template< class Archive > 
	void serialize( Archive& ar, const unsigned int version )
	{
		ar & sHost;
		ar & nPort;
		ar & sComponentName;
	}
};

} } // namespace Ubitrack::Drivers

#endif

