/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

/**
 * @ingroup drivers
 * @file
 * Implementation of a general conic class.
 *
 * @author Daniel Pustka <daniel.pustka@in.tum.de>
 */

#include <Ubitrack/Math/MatrixOperations.h>

#include "Conic.h"

namespace ublas = boost::numeric::ublas;

namespace Ubitrack { namespace Drivers { namespace PeopleTracking {

Conic::Conic( const Math::Vector< 3 >& pos, const Math::Vector< 3 >& size, const Math::Matrix< 3, 4 >& P )
{
	// generate quadric
	Math::Matrix< 4, 4 > Q;
	Q( 0, 0 ) = size( 0 ) * size( 0 ) - pos( 0 ) * pos( 0 );
	Q( 0, 1 ) = -pos( 0 ) * pos( 1 );
	Q( 0, 2 ) = -pos( 0 ) * pos( 2 );
	Q( 0, 3 ) = -pos( 0 );

	Q( 1, 0 ) = -pos( 0 ) * pos( 1 );
	Q( 1, 1 ) = size( 1 ) * size( 1 ) - pos( 1 ) * pos( 1 );
	Q( 1, 2 ) = -pos( 1 ) * pos( 2 );
	Q( 1, 3 ) = -pos( 1 );

	Q( 2, 0 ) = -pos( 0 ) * pos( 2 );
	Q( 2, 1 ) = -pos( 1 ) * pos( 2 );
	Q( 2, 2 ) = size( 2 ) * size( 2 ) - pos( 2 ) * pos( 2 );
	Q( 2, 3 ) = -pos( 2 );

	Q( 3, 0 ) = -pos( 0 );
	Q( 3, 1 ) = -pos( 1 );
	Q( 3, 2 ) = -pos( 2 );
	Q( 3, 3 ) = -1.0;
	
	// project to conic matrix
	Math::Matrix< 3, 4 > t( ublas::prod( P, Q ) );
	Math::Matrix< 3, 3 > C( ublas::prod( t, ublas::trans( P ) ) );
	m_matrix = Math::invert_matrix( C );
}

} } } // namespace Ubitrack::Drivers::PeopleTracking
