/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

/**
 * @ingroup drivers
 * @file
 * Implementation of a conic described by its start and end in each line.
 *
 * @author Daniel Pustka <daniel.pustka@in.tum.de>
 */

#include <iostream>
#include <algorithm>

#include <Ubitrack/Util/BlockTimer.h>

#include "OnScreenConic.h"

namespace Ubitrack { namespace Drivers { namespace PeopleTracking {

#if 1
	static Util::BlockTimer g_totalTimer( "conic generation total", "Ubitrack.Drivers.PeopleTracking" );
	static Util::BlockTimer g_resizeTimer( "conic generation lines", "Ubitrack.Drivers.PeopleTracking" );
	#define DO_TIME( name, x ) UBITRACK_TIME( x )
#else
	#define DO_TIME( name, x )
#endif

OnScreenConic::OnScreenConic( const Conic& C, int clipWidth, int clipHeight )
{
	DO_TIME( _timer_, g_totalTimer );
	
 	// compute begin and end in y direction
	std::pair< double, double > topBottom( C.getTopBottom() );
	m_y.begin = int( ceil( topBottom.first ) );
	m_y.end = int( floor( topBottom.second ) ) + 1;

	// clip
	m_y.begin = std::max( 0, std::min( clipHeight, m_y.begin ) );
	m_y.end = std::max( 0, std::min( clipHeight, m_y.end ) );
	
	// for each line, compute the left and right boundaries
	m_x.resize( m_y.end - m_y.begin );

	for ( int iY = m_y.begin; iY != m_y.end; iY++ )
	{
		std::pair< double, double > leftRight( C.getHLineIntersections( iY ) );

		m_x[ iY - m_y.begin ].begin = std::max( 0, std::min( clipWidth, int( leftRight.first ) ) );
		m_x[ iY - m_y.begin ].end = std::max( 0, std::min( clipWidth, int( leftRight.second ) + 1 ) );
	}
}

} } } // namespace Ubitrack::Drivers::PeopleTracking
