/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

/**
 * @ingroup drivers
 * @file
 * Declaration of a conic described by its start and end in each line.
 *
 * @author Daniel Pustka <daniel.pustka@in.tum.de>
 */


#ifndef __Ubitrack_Drivers_PeopleTracker_OnScreenConic_h_included__
#define __Ubitrack_Drivers_PeopleTracker_OnScreenConic_h_included__

#include <vector>

#include "Conic.h"

namespace Ubitrack { namespace Drivers { namespace PeopleTracking {

/**
 * Describes a conic in screen-pixels.
 * For each line in the region given by m_y, m_x describes the start and the end pixel of the conic in this line
 */
struct OnScreenConic
{
	/** constructs the region description of the conic from a 3x3 homogeneous matrix */
	OnScreenConic( const Conic& C, int clipWidth, int clipHeight );
	
	/** default constructor */
	OnScreenConic()
	{}

	/** begin-end pair of integers */
	struct BeginEnd
	{
		BeginEnd()
		{}
	
		BeginEnd( int _begin, int _end )
			: begin( _begin )
			, end( _end )
		{}
		
		/** the index of the start pixel */
		int begin;
		
		/** the index of the pixel after the end pixel */
		int end;
	};

	/** extension in y direction */
	BeginEnd m_y;
	
	/** for each line, starting from m_yExtension.begin, one BeginEnd struct that describes the extension in x direction */
	std::vector< BeginEnd > m_x;
};

} } } // namespace Ubitrack::Drivers::PeopleTracking

#endif
