/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */


/**
 * @ingroup driver_components
 * @file
 * RFID Stuff
 *
 * @author M.H. & B.B.
 */

#include "RFIDTracker.h"
#include "RFIDAntenna.h"

namespace Ubitrack { namespace Drivers { namespace RFID {

RFIDTracker::RFIDTracker()
{
}

RFIDTracker::~RFIDTracker()
{
}

bool RFIDTracker::EstimatePosition(const Measurement::Rotation& p_Rotation, const Measurement::PositionList& p_PositionList)
{
	for(std::vector < Math::Vector< 3 > >::iterator it = p_PositionList->begin(); it != p_PositionList->end(); it++)
	{
		m_position += *it;
	}
	m_position = (1.0/p_PositionList->size())*m_position;
	Math::Vector<3> lookat(0.0,0.0,0.60);	//60cm back translation , negative z is the direction of the antenna
	m_position -= (*p_Rotation)*lookat;
	
	return true;
}


} } } // namespace Ubitrack::Drivers::RFID