#include "SerialDriver.h"
#include <iostream>
#include "m5eCom.h"


M5eCom::M5eCom(int port, int baud)
{
	_SD = new SerialDriver();
	_SD->OpenPort(port, baud);

	_SD->ClearSerialPort(); // Clear the serial port in case there's any junk there
	
	
	//  Some level converters are powered off of serial pin #4.  It needs some
	//  time to charge up before it can communicate properly.  The one I have
	//  powers up in 5ms, but set to 30ms to be safe.  This sleep can be taken
	//  out if the program needs to be sped up.
	Sleep(30);
}

M5eCom::~M5eCom()
{
	_SD->ClearSerialPort();
	_SD->ClosePort();
	delete _SD;
}



bool M5eCom::secureCom( Message *hTX_Msg, Message *hRX_Msg, int wait, int timeout, int times)
{
	int sendcounter = 0;
	int readcounter = 0;

	sendMessage( hTX_Msg);
	Sleep(wait);
	while(readcounter <= times)
	{
		if(receiveMessageNTimes(hRX_Msg))
		{
			if(!checkResponse(hTX_Msg, hRX_Msg))
			{
			//	printf("Error! Status: %2X%2X | ",hRX_Msg->status[0],hRX_Msg->status[1]);
			//	std::cout << "Trials to Read: "<<readcounter<< "Trials to Send: "<<sendcounter<<" | ";
				return false;
			}
			else
			{
			//	std::cout << "S:"<<readcounter<<" | ";
			//	std::cout << "Trials to Read: "<<readcounter<< "Trials to Send: "<<sendcounter<<std::endl;
				return true;
			}
		}
		readcounter++;
		Sleep(timeout/times);
	}

	std::cout<<"Error! Nothing recived for command "<<hTX_Msg->opCode<<", might need bigger timeout!"<<std::endl;
	return false;
}

bool M5eCom::receiveMessageNTimes( Message *hMsg, int trials)
{
  s32 bytesRead;
  u8  soh;
  u8  i;
  u8  crc[2];
  bool retVal=false;
  int count=0;

  while(1)
  {
    bytesRead = _SD->ReadData(&soh, 1);
    soh &= 0xFF;

    if((bytesRead == 1) && (soh == 0xFF))
    {
	  retVal = true;
      break;
    }
	if(count > trials)
		return false;
	count++;
  }

  hMsg->crc = CRC_INIT;

  while(_SD->ReadData(&hMsg->dataLen, 1) == 0);
  while(_SD->ReadData(&hMsg->opCode, 1) == 0);
  while(_SD->ReadData(&hMsg->status[0], 1) == 0);
  while(_SD->ReadData(&hMsg->status[1], 1) == 0);

  for(i=0; i<hMsg->dataLen; i++)
  {
    while(_SD->ReadData(&hMsg->data[i], 1) == 0);
  }

  while(_SD->ReadData(&crc[0], 1) == 0);
  while(_SD->ReadData(&crc[1], 1) == 0);
  hMsg->crc = ((crc[0] & 0xFF) << 8) | (crc[1] & 0xFF);

  retVal = checkCrc(hMsg);
  if(retVal != true)
  {
		printf("Inavlid message received!\n");
  }
  return(retVal);
}

bool M5eCom::receiveMessage( Message *hMsg)
{
  s32 bytesRead;
  u8  soh;
  u8  i;
  u8  crc[2];
  bool retVal;

  int trials = 0;

  while(1)
  {
    bytesRead = _SD->ReadData(&soh, 1);
    soh &= 0xFF;

    if((bytesRead == 1) && (soh == 0xFF))
    {
	  retVal = true;
      break;
    }
	if(trials > 100)
		return false;
	trials++;
  }

  hMsg->crc = CRC_INIT;

  while(_SD->ReadData(&hMsg->dataLen, 1) == 0);
  while(_SD->ReadData(&hMsg->opCode, 1) == 0);
  while(_SD->ReadData(&hMsg->status[0], 1) == 0);
  while(_SD->ReadData(&hMsg->status[1], 1) == 0);

  for(i=0; i<hMsg->dataLen; i++)
  {
    while(_SD->ReadData(&hMsg->data[i], 1) == 0);
  }

  while(_SD->ReadData(&crc[0], 1) == 0);
  while(_SD->ReadData(&crc[1], 1) == 0);
  hMsg->crc = ((crc[0] & 0xFF) << 8) | (crc[1] & 0xFF);

  retVal = checkCrc(hMsg);
  if(retVal != true)
  {
		printf("Inavlid message received!\n");
  }
/*
  else
  {
	printf("Valid message received:\n");
    printf("  Data Length = %02X\n", hMsg->dataLen);
    printf("  OpCode      = %02X\n", hMsg->opCode);
    printf("  Status      = %02X %02X\n", hMsg->status[0], hMsg->status[1]);

    for(i=0; i<hMsg->dataLen; i++)
    {
      printf("  Data[%03d]   = %02X\n", i, hMsg->data[i]);
    }
    printf("  CRC         = %04X\n", hMsg->crc);
  }
*/

  return(retVal);
}


/** @fn void sendMessage(Message *hMsg)
  * @brief Sends a message to the DSP.
  *
  * @param hMsg   Pointer to the message to send.
  * @return none.
  */
void M5eCom::sendMessage( Message *hMsg)
{
  char  buf[MAX_PACKET_LEN];
  u8    i;
  u8    idx = 0;

  // Calculate the CRC
  hMsg->crc = calcCrcToDsp(hMsg);

  // Set up SOH in buffer
  buf[idx++] = (char)0xFF;

  // Send data length
  buf[idx++] = hMsg->dataLen & 0xFF;

  // Send OpCode
  buf[idx++] = hMsg->opCode;

  // Send Data
  for(i=0; i<hMsg->dataLen; i++)
  {
    buf[idx++] = hMsg->data[i] & 0xFF;
  }

  // Send CRC
  buf[idx++] = hMsg->crc >> 8;
  buf[idx++] = hMsg->crc & 0xFF;

  // Send the data to the serial port
  _SD->SendData(buf, idx);
}


// verification (matching tx-rx , good status etc)

bool M5eCom::checkOpcode(const Message *hTX_Msg, const Message *hRX_Msg)
{
	return (hTX_Msg->opCode == hRX_Msg->opCode);
}

bool M5eCom::checkStatus(const Message *hMsg)
{
	return ((hMsg->status[0] == 0x00) && (hMsg->status[1] == 0x00));
}

bool M5eCom::checkResponse(const Message *hTX_Msg, const Message *hRX_Msg)
{
	return (checkOpcode(hTX_Msg,hRX_Msg) && checkStatus(hRX_Msg));
}




/** @fn bool checkCrc(Message *hMsg)
  * @brief Simple utility to validate CRC of message received from DSP.
  *
  * @param hMsg   Pointer to message to calculate CRC of.
  * @return true if CRC is correct.
  */
bool M5eCom::checkCrc(const Message *hMsg)
{
  u16 calcCrc;

  calcCrc = calcCrcFromDsp(hMsg);

  return(calcCrc == hMsg->crc);
}


/** @fn u16 calcCrcToDsp(const Message *hMsg)
  * @brief Calculates the CRC on a message sent to the DSP.
  *
  *         The CRC calculations are slightly different between messages sent
  *         to the DSP and messages received from the DSP because of the status
  *         field.  The status field is not used when transmitting to the DSP.
  *
  * @param hMsg   Pointer to message to calculate CRC of.
  * @return CRC for the message object.
  */
u16 M5eCom::calcCrcToDsp(const Message *hMsg)
{
  u16 calcCrc = CRC_INIT;
  u8  i;

  CRC_calcCrc8(&calcCrc, CCITT_CRC_POLY, hMsg->dataLen);
  CRC_calcCrc8(&calcCrc, CCITT_CRC_POLY, hMsg->opCode);


  for(i=0; i<hMsg->dataLen; i++)
  {
    CRC_calcCrc8(&calcCrc, CCITT_CRC_POLY, hMsg->data[i]);
  }

  return(calcCrc);
}


/** @fn u16 calcCrcFromDsp(const Message *hMsg)
  * @brief Calculates the CRC on a message received from the DSP.
  *
  *         The CRC calculations are slightly different between messages sent
  *         to the DSP and messages received from the DSP because of the status
  *         field.  The status field is not used when transmitting to the DSP.
  *
  * @param hMsg   Pointer to message to calculate CRC of.
  * @return CRC for the message object.
  */
u16 M5eCom::calcCrcFromDsp(const Message *hMsg)
{
  u16 calcCrc = CRC_INIT;
  u8  i;

  CRC_calcCrc8(&calcCrc, CCITT_CRC_POLY, hMsg->dataLen);
  CRC_calcCrc8(&calcCrc, CCITT_CRC_POLY, hMsg->opCode);
  CRC_calcCrc8(&calcCrc, CCITT_CRC_POLY, hMsg->status[0]);
  CRC_calcCrc8(&calcCrc, CCITT_CRC_POLY, hMsg->status[1]);

  for(i=0; i<hMsg->dataLen; i++)
  {
    CRC_calcCrc8(&calcCrc, CCITT_CRC_POLY, hMsg->data[i]);
  }

  return(calcCrc);
}

// finally compute CRC...

void M5eCom::CRC_calcCrc8(u16 *crcReg, u16 poly, u16 u8Data)
{
	u16 i;
	u16 xorFlag;
	u16 bit;
	u16 dcdBitMask = 0x80;

	for(i=0; i<8; i++)
	{
		// Get the carry bit.  This determines if the polynomial should be xor'd
		//	with the CRC register.
		xorFlag = *crcReg & 0x8000;
		
		// Shift the bits over by one.
		*crcReg <<= 1;
		
		// Shift in the next bit in the data byte
		bit = ((u8Data & dcdBitMask) == dcdBitMask);
		*crcReg |= bit;
		
		// XOR the polynomial
		if(xorFlag)
		{
			*crcReg = *crcReg ^ poly;
		}
		
		// Shift over the dcd mask
		dcdBitMask >>= 1;	
	}
}
