#ifndef __M5E_READER_H__
#define __M5E_READER_H__

#include <Ubitrack/Math/Scalar.h>
#include <vector>

struct Message;
class M5eCom;
class SerialDriver;

#define EPC_GEN2 5

enum Region
{
	NorthAmerica = 1,
	Europe = 2,
	Korea =3
};

enum BaudRate
{
	baud_9600 = 488,
	baud_19200 = 244,
	baud_38400 = 122,
	baud_57600 = 84,
	baud_115200 = 41,
	baud_230400 = 20,
	baud_460800 = 10
};

enum AntennaPort
{
	port_1 = 1,
	port_2 = 2
};


struct ReaderParameter
{
	BaudRate baudrate;
	Region region;
	
	int tx_read_power;
	int tx_write_power;

	AntennaPort antennaRXPort;
	AntennaPort antennaTXPort;

	int protocol;
	int power;
};

struct Messages
{
	Message* tx;
	Message* rx;
};

class M5eReader
{
public:
	M5eReader(int comPort, Region region, BaudRate baud, AntennaPort antennaRXPort, AntennaPort antennaTXPort);
	~M5eReader();

	bool PrintVisibleTags(int timeout=250); //millisec
	bool GetVisibleTags(std::vector<Ubitrack::Math::Scalar<unsigned long> >& tagIDs, int timeout=250);
	bool SetReadTXPower(int centi_dBm);		//centi-dBm 500-3000 / 5-30dBm
	//bool SetWriteTXPower();

private:
	bool StartBootloader();
	bool BootFirmware();
	bool SetBaudrate(BaudRate baud);
	bool SetTagProtocol(int protocol = EPC_GEN2);	// currently m5e supports only epc gen2
	bool SetAntennaPort(AntennaPort antennaRXPort, AntennaPort antennaTXPort);
	bool SetRegion(Region region);

	int				_comPort;
	ReaderParameter _readerParameter;
	Messages		_msg;
	M5eCom*			_m5eCom;
};

#endif // __M5E_READER_H__