/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

#include "Additions.h"
#include <boost/numeric/ublas/io.hpp>
#include <iostream>

namespace ublas = boost::numeric::ublas;

long long abs( long long a)
{
	if( a <  0LL  )
		return -a;
	else return a;
}

TraceContainer::TraceContainer()
{
	m_new = true;
}

void TraceContainer::addPoint( Ubitrack::Measurement::Position2D p )
{
	m_trace.push_back( p );

	if( m_new )
	{
		m_new = false;
		m_movement = Ubitrack::Math::Vector< 4 >( (*p)( 0 ), (*p)( 0 ), (*p)( 1 ), (*p)( 1 ) );
	}
	else
	{
		if( (*p)( 0 ) < m_movement( 0 ) )
			m_movement( 0 ) = (*p)( 0 );
		if( (*p)( 0 ) > m_movement( 1 ) )
			m_movement( 1 ) = (*p)( 0 );
		if( (*p)( 1 ) < m_movement( 2 ) )
			m_movement( 2 ) = (*p)( 1 );
		if( (*p)( 1 ) > m_movement( 3 ) )
			m_movement( 3 ) = (*p)( 1 );
	}
}

const Ubitrack::Math::Vector< 2 > & TraceContainer::at(int index) const
{
	return *m_trace.at( index );
}

double TraceContainer::getMovement() const
{
	return ublas::norm_2( Ubitrack::Math::Vector< 2 >( m_movement( 1 ) - m_movement( 0 ), m_movement( 3 ) - m_movement( 2 ) ) );
}

long long TraceContainer::getTime(int index) const
{
	return m_trace.at( index ).time();
}

unsigned int TraceContainer::size() const
{
	return m_trace.size();
}