/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

#ifndef __ADDITIONS_H_INCLUDED__
#define __ADDITIONS_H_INCLUDED__

#include <Ubitrack/config.h>
#include <Ubitrack/Measurement/Measurement.h>

#include <map>

long long abs( long long a);

class TraceContainer
{
	public:
		TraceContainer();
		void addPoint( Ubitrack::Measurement::Position2D p );
		const Ubitrack::Math::Vector< 2 > & at( int index ) const;
		long long getTime( int index ) const;
		double getMovement() const;
		unsigned int size() const;

	private:
		bool m_new;
		//the actual trace
		std::vector< Ubitrack::Measurement::Position2D > m_trace;

		//movement of points, 4D vector: x_min, x_max, y_min, y_max
		Ubitrack::Math::Vector< 4 > m_movement;
};

std::vector< std::pair< unsigned int, unsigned int > > findBody( const std::vector< Ubitrack::Math::Vector< 3 > > & points, const std::vector< Ubitrack::Math::Vector< 3 > > & body, const double & error );




#endif
