/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

#include "RBCalibration.h"

static log4cpp::Category& logger( log4cpp::Category::getInstance( "Drivers.RigidBodyTracking" ) );

using namespace Ubitrack;
using namespace Ubitrack::Dataflow;

// shortcuts to namespaces
namespace ublas = boost::numeric::ublas;

namespace Ubitrack { namespace Drivers {	
	
/** constructor, creates thread */
RBCalibrationModule::RBCalibrationModule( const Dataflow::SingleModuleKey& key, boost::shared_ptr< Graph::UTQLSubgraph >, FactoryHelper* pFactory )
	: ModuleBaseC( key, pFactory )
	, m_nCams( 0 )
	, m_eMax( 0.0 )
	, m_nMin( 0 )
	, m_tMax( 0 )
{
	comps = getAllComponents();
}

//destructor
RBCalibrationModule::~RBCalibrationModule()
{}

void RBCalibrationModule::createCam( double eMax, unsigned int nMin, long long tMax )
{
	m_eMax = eMax;
	m_nMin = nMin;
	m_tMax = tMax;
	m_nCams++;
}

/** destroys a cam */
void RBCalibrationModule::destroyCam()
{
	m_nCams--;
}

/** start the calibration */
void RBCalibrationModule::startCalibration()
{
	comps = getAllComponents();
	ComponentList::iterator it = comps.begin();
	ComponentList::iterator it2 = comps.begin();
	it2++;

	Math::Matrix< 3, 3 > id = ublas::identity_matrix< double >( 3 );

	while( it2 != comps.end() )
	{
		Math::Matrix< 3, 3 > fM = findFM( (*it)->getList(), (*it2)->getList() );

		if( m_ok )
		{
			Math::Pose p2_ = Calibration::poseFromFundamentalMatrix( fM, m_fromPoints.at( 0 ), m_toPoints.at( 0 ), (*it)->getIntrinsics(), (*it2)->getIntrinsics() );
			Math::Pose p2 = Math::Pose( p2_.rotation(), p2_.translation() /* * 3.82 / ublas::norm_2( p2_.translation()  ) */ );

			Math::Pose tmp = Math::Pose( Math::Quaternion( id ), Math::Vector< 3 >( 0.0, 0.0, 0.0 ) );
			(*it)->setPose( tmp );
			(*it2)->setPose( p2 );
		}
		it2++;
	}
}

} } // namespace Ubitrack::Driver
