/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

#include "RBTracking.h"

static log4cpp::Category& logger( log4cpp::Category::getInstance( "Drivers.RigidBodyTracking" ) );

using namespace Ubitrack;
using namespace Ubitrack::Dataflow;

// shortcuts to namespaces
namespace ublas = boost::numeric::ublas;

namespace Ubitrack { namespace Drivers {	
	
/** constructor, creates thread */
RBTrackingModule::RBTrackingModule( const Dataflow::SingleModuleKey& key, boost::shared_ptr< Graph::UTQLSubgraph >, FactoryHelper* pFactory )
	: ModuleBaseT( key, pFactory )
	, m_nCams( 0 )
{
	comps = getAllComponents();
}

//destructor
RBTrackingModule::~RBTrackingModule()
{}

void RBTrackingModule::createCam( double e, long long t )
{	
	m_eMax = e;
	m_tMax = t;
	m_nCams++;
}

/** destroys a cam */
void RBTrackingModule::destroyCam()
{
	m_nCams--;
}

/** start the calibration */
void RBTrackingModule::compute()
{
}

void RBTrackingModule::sendPoints()
{
	
	ComponentList::iterator it = comps.begin();
	ComponentList::iterator it2 = comps.begin();
	it2++;

	std::vector< Math::Vector< 3 > > list;

	std::vector< Math::Vector< 3 > > body;

	body.push_back( Math::Vector< 3 >( 0.0, 0.0, 0.0 ) );
	body.push_back( Math::Vector< 3 >( 0.168252344, 0.0, 0.0 ) );
	body.push_back( Math::Vector< 3 >( 0.036370005, 0.092717175, 0.0 ) );
	body.push_back( Math::Vector< 3 >( 0.058814988, 0.065122907, 0.078185440 ) );
	body.push_back( Math::Vector< 3 >( 0.098418726, 0.097669257, 0.041206136 ) );

	while( it2 != comps.end() )
	{
		Measurement::PositionList2  d1 = ( *it )->getData();
		Measurement::PositionList2  d2 = ( *it2 )->getData();

		long long diff = d1.time() - d2.time();


		if( abs( diff ) < m_tMax )
		{

			list = Calibration::reconstruct3DPoints( *d1, *d2, ( *it )->getPM(), ( *it2 )->getPM(), ( *it2 )->getFM() );
			if( list.size() > 0 )
			{
				std::vector< Math::Vector< 3 > > other;
				std::vector< Math::Vector< 3 > > ofBody;

				std::vector< std::pair< unsigned int, unsigned int > > pair = findBody( list, body, m_eMax );
				
				/*Measurement::PositionList tmp( Ubitrack::Measurement::now(), list );
				if( list.size() > 0 )
					( *it )->sendPoints( tmp );*/

				for( unsigned int i=0; i < list.size(); i++ )
				{
					bool add = true;
					for( unsigned int j=0; j < pair.size(); j++)
						if( pair.at( j ).second == i )
							add = false;
					if( add )
						other.push_back( list.at( i ) );
				}

				for( unsigned int j=0; j < pair.size(); j++)
					ofBody.push_back( list.at( pair.at( j ).second ) );

				if( other.size() > 0 ){
					Measurement::PositionList tmp( Ubitrack::Measurement::now(), other );
					( *it2 )->sendPoints( tmp );
				}
				if( ofBody.size() > 0 ){
					Measurement::PositionList tmp2( Ubitrack::Measurement::now(), ofBody );
					( *it )->sendPoints( tmp2 );
				}
			}
		}

		it2++;
	}
}

} } // namespace Ubitrack::Driver