/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

/**
 * @ingroup tracking_algortihms
 * @file
 * Shows a trace with a number
 *
 * @author Daniel Muhra <muhra@in.tum.de>
 */

#ifndef __CALIBRATION_SHOW_TRACE_H_INCLUDED__
#define __CALIBRATION_SHOW_TRACE_H_INCLUDED__

#include <Ubitrack/config.h>
#include <Ubitrack/Measurement/Measurement.h>
#include <Ubitrack/Calibration/Munkres.h>

#include "Additions.h"

#include <cv.h>
#include <Vision/Image.h>
#include <sstream>

namespace Ubitrack { namespace Calibration {

class ShowTrace
{
public:
	ShowTrace();
	ShowTrace( double dMin, double dMax );
    
	/** Method that computes the result. */
	void enlist( Measurement::PositionList2 & data, std::map< int, TraceContainer > & traceList );
	
	Vision::ImageMeasurement sendImage( Measurement::Timestamp t );
	
private:
	CvFont font;
	Vision::Image* blackImage;
	
	//counter for point id's
	int m_count;

	//minimal and maximal distance between two points
	double m_dMax;
	double m_dMin;

	//sizes of previous and current points
	bool m_first;
	std::vector< Math::Vector< 2 > > m_prev;
	
	//matching of current points to previous points 
	std::vector< unsigned int > m_matchList;

	//holds the id's for the current/previous points
	std::vector< int > m_id;

	//was the point matched the last time?
	std::vector< bool > m_wasMatched;
};

} } // namespace Ubitrack::Calibration

#endif
