/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */


/**
 * @ingroup driver_components
 * @author Florian Echtler <echtler@in.tum.de>
 */

#ifndef _AUDIOSTREAM_H_
#define _AUDIOSTREAM_H_

#ifndef WIN32

extern "C" {
	#include <libavformat/avformat.h>
	#include <libswscale/swscale.h>
}

class AudioStream {

	public:

		/** constructor: add an audio output stream */
		AudioStream( AVFormatContext* fcon, int codec_id );

		~AudioStream();

		void open();
		void write_audio_frame( );
		double get_pts();

	protected:

		void try_ioctl( int cmd, int val );

		void get_audio_frame(int16_t *samples, int frame_size, int nb_channels);

		int16_t* samples;
		uint8_t* audio_outbuf;
		int audio_outbuf_size;
		int audio_input_frame_size;
		int dev_handle;

		AVStream* st;
		AVFormatContext* oc;

};

#else

// dummy for WIN32
class AudioStream 
{
public:
	AudioStream( AVFormatContext* fcon, int codec_id )
	{}

	void open()
	{}
	
	void write_audio_frame()
	{}

	double get_pts()
	{ return 0; }
};

#endif

#endif

