/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */


/**
 * @ingroup driver_components
 * @author Florian Echtler <echtler@in.tum.de>
 */

#include "DataStream.h"

#include <stdexcept>


/** constructor: add a data output stream */
DataStream::DataStream( AVFormatContext* fcon, int codec_id )
{
	AVCodecContext *c;

	oc = fcon;

	st = av_new_stream( oc, 1 );
	if (!st) throw std::runtime_error( "Could not allocate data stream." );

	c = st->codec;
	c->codec_id = CODEC_ID_NONE;//(CodecID)codec_id;
	c->codec_type = CODEC_TYPE_DATA;
}

void DataStream::open() {

	#if 0 
	AVCodecContext* c = st->codec;

	/* find the audio encoder */
	AVCodec* codec = avcodec_find_encoder( c->codec_id );
	if (!codec) throw std::runtime_error( "Data codec not found." );

	/* open it */
	if (avcodec_open( c, codec ) < 0) throw std::runtime_error( "Could not open data codec." );
	#endif
}


DataStream::~DataStream()
{
	//avcodec_close(st->codec);
}


void DataStream::write_data_frame( unsigned char* data )
{
	AVCodecContext *c;
	AVPacket pkt;

	av_init_packet( &pkt );

	c = st->codec;

	//pkt.pts = av_rescale_q( c->coded_frame->pts, c->time_base, st->time_base );
	pkt.pts = av_rescale_q( 0, c->time_base, st->time_base );
	pkt.stream_index = st->index;
	pkt.data = data;
	pkt.size = strlen((const char*)data);

	/* write the compressed frame in the media file */
	if (av_write_frame( oc, &pkt ) != 0) throw std::runtime_error( "Error while writing data frame." );
}

double DataStream::get_pts()
{
	return (double)st->pts.val * st->time_base.num / st->time_base.den;
}

