/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

#ifndef __InventorObject_h_INCLUDED__
#define __InventorObject_h_INCLUDED__

#include "RenderModule.h"

#include <Inventor/nodes/SoNodes.h>

namespace Ubitrack { namespace Drivers {


/**
 * @ingroup driver_components
 * Component for Inventor render objects.
 * Provides a push-in port for poses and a push-in port for images.
 * This file was copied from X3DObject.h and modified to fit.
 * @author Nicolas Heuser <heuser@in.tum.de>
 */
class InventorObject
	: public VirtualObject
{
public:

	/**
	 * Constructor
	 * @param name edge name
	 * @param config component configuration
	 * @param componentKey the unique identifier for this component
	 * @param pModule parent object
	 */
	InventorObject( const std::string& name, boost::shared_ptr< Graph::UTQLSubgraph > subgraph, 
		const VirtualObjectKey& componentKey, VirtualCamera* pModule );

	virtual ~InventorObject();


	/** render the object, if up-to-date tracking information is available */
	virtual void draw( Measurement::Timestamp& t, int parity );

	virtual bool hasWaitingEvents();

protected:

	/**
	 * callback from Pose port
	 * @param pose current transformation
	 * @param redraw trigger a redraw? (only if called from push-input)
	 */
	void poseIn( const Ubitrack::Measurement::Pose& pose, int redraw );

	// pose input
	PushConsumer< Ubitrack::Measurement::Pose > m_push;
	PullConsumer< Ubitrack::Measurement::Pose > m_pull;

	double m_pose[16];
	boost::mutex m_poseLock;

	// Inventor root node
	SoNode*	m_rootNode;
	
};

} } // namespace Ubitrack::Drivers

#endif

