/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

#include "Projection.h"
#include <Ubitrack/Calibration/Projection.h>

namespace Ubitrack { namespace Drivers {

Projection::Projection( const std::string& name, boost::shared_ptr< Graph::UTQLSubgraph > subgraph, 
	const VirtualObjectKey& componentKey, VirtualCamera* pModule )
	: VirtualObject( name, subgraph, componentKey, pModule )
	, m_push( "PushInput", *this, boost::bind( &Projection::inputIn, this, _1, 1 ) )
	, m_pull( "PullInput", *this)
{
}

/** render the object */
void Projection::draw( Measurement::Timestamp& time, int parity )
{
	if (m_pull.isConnected())
		inputIn( m_pull.get(time), 0);

	LOG4CPP_TRACE( logger, "Updating projection matrix to:" << std::endl << m_projection );
	glMatrixMode( GL_PROJECTION );
	glLoadMatrixd( m_projection.content() );
	glMatrixMode( GL_MODELVIEW );
}

/**
 * callback from Pose input port
 * @param pose input pose
 * @param redraw trigger a redraw? (only if called from push-input)
 */
void Projection::inputIn( const Measurement::Matrix4x4& m, int redraw )
{
	m_lastUpdateTime = m.time();
	m_projection = *(m.get());
	// the pose has changed, so redraw the world
	if (redraw) m_pModule->invalidate();
}

} } // namespace Ubitrack::Drivers

