/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

/**
 * @file
 * Implementation of a drop shadow component
 *
 * @author Daniel Pustka <daniel.pustka@in.tum.de>
 */

#include "WorldFrame.h"
//#define CONE1 200;
//#define CONE 270;

namespace Ubitrack { namespace Drivers {	
WorldFrame::WorldFrame( const std::string& name, boost::shared_ptr< Graph::UTQLSubgraph > subgraph,
	const VirtualObjectKey& componentKey, VirtualCamera* pModule )
	: TrackedObject( name, subgraph, componentKey, pModule )
	, m_width( 1 )
	, m_height( 1 )
	, m_distance( 1 )
	, m_bInitialized( false )
{
	// read parameters
	Graph::UTQLSubgraph::NodePtr objectNode = subgraph->getNode( "Object" );
	if ( !objectNode )
		UBITRACK_THROW( "No Object node in RenderModule configuration" );

	objectNode->getAttributeData( "shadowWidth", m_width );
	objectNode->getAttributeData( "shadowHeight", m_height );
	objectNode->getAttributeData( "distance", m_distance );
	time=0;
}


WorldFrame::~WorldFrame()
{
	// TODO: delete texture
}


void WorldFrame::draw3DContent( Measurement::Timestamp& t, int )
{             

     int CONE1=200;
     int CONE2=270;
	 double length_X=m_distance/3;
	 double length_Y=m_distance/3;
	 double length_Z=m_distance/5;

	
	 glRotatef(-45,0,1.0,0);
	 glRotatef(15,1.0,0,0);
	 glRotatef(-15,0,0,1.0);
//x
     glColor4f( 1.0, 0.0, 0.0, 1.0 );
	 glBegin(GL_LINES);
	     glVertex3f(0,0,-length_X);
	     glVertex3f(0,0,length_X);
	 glEnd();

	 glPushMatrix();
	 glTranslatef(0,0,length_X);
	 glutWireCone(m_distance/CONE1,m_distance/CONE2,10,10);
	 glPopMatrix();
//y
     glColor4f( 0.0, 1.0, 0.0, 1.0 );
	 glBegin(GL_LINES);
	     glVertex3f(-length_Y,0.0f,0.0f);
	     glVertex3f(length_Y,0.0f,0.0f);
	 glEnd();

	 glPushMatrix();
	 	 glTranslatef(length_Y,0.0,0.0f);
	 	glRotatef(90.0f,0.0f,90.0f,0.0f);
	 	 glutWireCone(m_distance/CONE1,m_distance/CONE2,10.0f,10.0f);
	 	 glPopMatrix();
//z
     glColor4f( 0.0, 0.0, 1.0, 1.0 );
	 glBegin(GL_LINES);
	     glVertex3f(0.0f,-length_Z,0.0f);
	     glVertex3f(0.0f,length_Z,0.0f);
	 glEnd();

	 glPushMatrix();
	 glTranslatef(0.0f,length_Z,0.0f);
	 glRotatef(90.0f,-1.0f,0.0f,0.0f);
	 glutWireCone(m_distance/CONE1,m_distance/CONE2,10.0f,10.0f);
	 glPopMatrix();

//draw the dash lines
	 glLineWidth (0.5);
     glColor4f( 1.0, 1.0, 1.0, 1.0 );
	 for(float i=(float)length_X/m_width;i<length_X;i+=(float)length_X/m_width){
		 glEnable(GL_LINE_STIPPLE);
		 glLineStipple (0, 0x0F0F);
		 glBegin(GL_LINES);
				 glVertex3f(i,0.0f,-length_X);
		 	     glVertex3f(i,0.0f,length_X);
		 glEnd();
	 }
	  for(float i=-(float)length_X/m_width;i>-length_X;i-=(float)length_X/m_width){
		 glEnable(GL_LINE_STIPPLE);
		 glLineStipple (0, 0x0F0F);
		 glBegin(GL_LINES);
				 glVertex3f(i,0.0f,-length_X);
		 	     glVertex3f(i,0.0f,length_X);
		 glEnd();
	 }
	 
	 for(float i=(float)length_Y/m_height;i<length_Y;i+=(float)length_Y/m_height){
		 glBegin(GL_LINES);
		 				 glVertex3f(-length_Y,0.0,i);
		 		 	     glVertex3f(length_Y,0.0,i);
		 glEnd();
	 }
	 for(float i=-(float)length_Y/m_height;i>-length_Y;i-=(float)length_Y/m_height){
		 glBegin(GL_LINES);
		 				 glVertex3f(-length_Y,0.0,i);
		 		 	     glVertex3f(length_Y,0.0,i);
		 glEnd();
	 }
	 glDisable(GL_LINE_STIPPLE);
     
}

} } // namespace Ubitrack::Drivers


