/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
 
#ifndef _MESASWISSRANGER_H_
#define _MESASWISSRANGER_H_


//OpenCV
#include <cv.h>
#include <highgui.h>

//SR4000 MESA
#include <definesSR.h>
#include <libusbSR.h>
#pragma comment (lib, "libusbSR" )


#include <boost/thread.hpp>


#include <Ubitrack/Dataflow/PushSupplier.h>
#include <Ubitrack/Dataflow/ComponentFactory.h>
#include <Ubitrack/Dataflow/Component.h>
#include <Ubitrack/Measurement/Measurement.h>
#include <Ubitrack/Util/Exception.h>
#include <Ubitrack/Util/OS.h>
#include <Vision/Image.h>


namespace Ubitrack { namespace Drivers
{

class SwissRanger : public Ubitrack::Dataflow::Component
{
	public:
		SwissRanger( const std::string& sName, boost::shared_ptr< Graph::UTQLSubgraph > pConfig );
		~SwissRanger( );
		
		void start( );
		void stop( );
		
	protected:

		/** Basic structure for use with the camera*/
		CMesaDevice* srCam;
	
		/** Flag - how many differnetn types of image can be aquired*/
		int m_img_types;
		
		/** Height of the image*/
		int m_img_height;
		
		/** Widthof the image*/
		int m_img_width;
		
		/**Flag - shows wheter the thread is running or not */
		bool m_bStop;
		
		/** main processing loop */
		void threadProc( );

		boost::shared_ptr < boost::thread > m_thread;

		Ubitrack::Dataflow::PushSupplier< Vision::ImageMeasurement > m_portOut1; // Distance Image
		Ubitrack::Dataflow::PushSupplier< Vision::ImageMeasurement > m_portOut2; //Amplitude Image
};

} } //Ubitrack::Drivers

#endif //_MESASWISSRANGER_H_