using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.IO;
using System.Net.Sockets;
using System.Net;
// Include the Ubisense COM interface
using Ubisense.Com;

namespace UbisenseSimpleLogger
{
	/// <summary>
	/// Example Ubisense application which logs basic location data for selected
	/// objects to files. All objects of the specified class are displayed in a
	/// list and the user can select any number of these and start logging. A
	/// comma separated value (csv) file is created for each object with the 
	/// following fields:
	///   Time stamp
	///   x (meters)
	///   y (meters)
	///   z (meters)
	///   distance since last record (meters)
	///   time since last record (seconds)
	/// The data can be filtered by specifying a minimum time and/or a minimum
	/// distance between exported values. Both values must be exceeded before a
	/// record is written. These values can be changed while files are being written
	/// and the change will take effect immediately.
	/// 
	/// The export occurs in the method exportData: this method could be modified
	/// to write data to a database or other external system, with the same filtering
	/// process being applied.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
        private System.Windows.Forms.Button button1;
        private System.Windows.Forms.ListBox listBox1;
        private Label label3;
        private TextBox textBoxPosX;
        private TextBox textBoxPosY;
        private TextBox textBoxPosZ;
        private Label label4;
        private TextBox textBoxReceiverAddress;
        private TextBox textBoxReceiverPort;
        private TextBox textBoxComponentName;
        private Label label1;
        private Label label2; 

		// Application variables
		private CNameClient nameClient;
		private CLocationClient locationClient;
        private Hashtable objectTable;
        private string m_selectedPerson;
        private System.Globalization.CultureInfo m_culture =
            new System.Globalization.CultureInfo("en-US");

        // Network stuff
        private IPEndPoint m_endPoint;
        private Socket m_socket;
        private string m_componentName;

        public delegate void SetFormCoordsD(double x, double y, double z);
        public SetFormCoordsD SetFormCoords;

		public Form1()
		{
            Application.CurrentCulture = m_culture;
            SetFormCoords = new SetFormCoordsD(FormThreadSetFormCoords);

			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			// Initialize application variables
			nameClient = new CNameClientClass();
			locationClient = new CLocationClientClass();

			// Get the names of all location cells and connect to them all
			// This assumes that this is a relatively small system - for a 
			// large system you would typically split the location cell handling
			// between multiple machines
			System.Array cellNames;
			cellNames = (System.Array)locationClient.GetAllCells();
			foreach (string cellName in cellNames)
			{
				locationClient.LoadCell(cellName);
				Console.WriteLine("Loading cell {0}", cellName);
			}

			// Get the names of all objects of type "Person" (or whichever other class
			// is specified below), and add these to the list box
			objectTable = new Hashtable();
			System.Array personArray;
			personArray = (System.Array)nameClient.GetAllNamed("Person");
			foreach(System.Array person in personArray)
			{
				// The person object is an array with index 0 being the object identifier
				// and index 1 being the object name
				string personName = (string)person.GetValue(1);
				listBox1.Items.Add(personName);
				// Also store reference between internal object id and external name in a 
				// Hashtable
				objectTable[(string)person.GetValue(0)] = personName;
			}

            if (personArray.Length > 0)
                listBox1.SelectedIndex = 0;

			// Specify the method to be called by the OnMove event
			locationClient.OnMove +=new _ILocationClientEvents_OnMoveEventHandler(
				locationClient_OnMove);
		}

        protected void FormThreadSetFormCoords(double x, double y, double z)
        {
            textBoxPosX.Text = x.ToString("f3");
            textBoxPosY.Text = y.ToString("f3");
            textBoxPosZ.Text = z.ToString("f3");
        }

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.button1 = new System.Windows.Forms.Button();
            this.listBox1 = new System.Windows.Forms.ListBox();
            this.label3 = new System.Windows.Forms.Label();
            this.textBoxPosX = new System.Windows.Forms.TextBox();
            this.textBoxPosY = new System.Windows.Forms.TextBox();
            this.textBoxPosZ = new System.Windows.Forms.TextBox();
            this.label4 = new System.Windows.Forms.Label();
            this.textBoxReceiverAddress = new System.Windows.Forms.TextBox();
            this.textBoxReceiverPort = new System.Windows.Forms.TextBox();
            this.textBoxComponentName = new System.Windows.Forms.TextBox();
            this.label1 = new System.Windows.Forms.Label();
            this.label2 = new System.Windows.Forms.Label();
            this.SuspendLayout();
            // 
            // button1
            // 
            this.button1.Location = new System.Drawing.Point(8, 274);
            this.button1.Name = "button1";
            this.button1.Size = new System.Drawing.Size(104, 24);
            this.button1.TabIndex = 14;
            this.button1.Text = "Apply";
            this.button1.Click += new System.EventHandler(this.button1_Click);
            // 
            // listBox1
            // 
            this.listBox1.Location = new System.Drawing.Point(8, 8);
            this.listBox1.Name = "listBox1";
            this.listBox1.Size = new System.Drawing.Size(321, 134);
            this.listBox1.TabIndex = 3;
            // 
            // label3
            // 
            this.label3.AutoSize = true;
            this.label3.Location = new System.Drawing.Point(5, 216);
            this.label3.Name = "label3";
            this.label3.Size = new System.Drawing.Size(90, 13);
            this.label3.TabIndex = 6;
            this.label3.Text = "Component name";
            // 
            // textBoxPosX
            // 
            this.textBoxPosX.Location = new System.Drawing.Point(143, 242);
            this.textBoxPosX.Name = "textBoxPosX";
            this.textBoxPosX.ReadOnly = true;
            this.textBoxPosX.Size = new System.Drawing.Size(58, 20);
            this.textBoxPosX.TabIndex = 7;
            this.textBoxPosX.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
            // 
            // textBoxPosY
            // 
            this.textBoxPosY.Location = new System.Drawing.Point(207, 242);
            this.textBoxPosY.Name = "textBoxPosY";
            this.textBoxPosY.ReadOnly = true;
            this.textBoxPosY.Size = new System.Drawing.Size(58, 20);
            this.textBoxPosY.TabIndex = 8;
            this.textBoxPosY.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
            // 
            // textBoxPosZ
            // 
            this.textBoxPosZ.Location = new System.Drawing.Point(271, 242);
            this.textBoxPosZ.Name = "textBoxPosZ";
            this.textBoxPosZ.ReadOnly = true;
            this.textBoxPosZ.Size = new System.Drawing.Size(58, 20);
            this.textBoxPosZ.TabIndex = 9;
            this.textBoxPosZ.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
            // 
            // label4
            // 
            this.label4.AutoSize = true;
            this.label4.Location = new System.Drawing.Point(5, 245);
            this.label4.Name = "label4";
            this.label4.Size = new System.Drawing.Size(80, 13);
            this.label4.TabIndex = 10;
            this.label4.Text = "Current position";
            // 
            // textBoxReceiverAddress
            // 
            this.textBoxReceiverAddress.Location = new System.Drawing.Point(143, 158);
            this.textBoxReceiverAddress.Name = "textBoxReceiverAddress";
            this.textBoxReceiverAddress.Size = new System.Drawing.Size(186, 20);
            this.textBoxReceiverAddress.TabIndex = 11;
            this.textBoxReceiverAddress.Text = "131.159.10.48";
            // 
            // textBoxReceiverPort
            // 
            this.textBoxReceiverPort.Location = new System.Drawing.Point(143, 186);
            this.textBoxReceiverPort.Name = "textBoxReceiverPort";
            this.textBoxReceiverPort.Size = new System.Drawing.Size(186, 20);
            this.textBoxReceiverPort.TabIndex = 12;
            this.textBoxReceiverPort.Text = "21844";
            // 
            // textBoxComponentName
            // 
            this.textBoxComponentName.Location = new System.Drawing.Point(143, 213);
            this.textBoxComponentName.Name = "textBoxComponentName";
            this.textBoxComponentName.Size = new System.Drawing.Size(186, 20);
            this.textBoxComponentName.TabIndex = 13;
            this.textBoxComponentName.Text = "Ubisense1";
            // 
            // label1
            // 
            this.label1.AutoSize = true;
            this.label1.Location = new System.Drawing.Point(5, 163);
            this.label1.Name = "label1";
            this.label1.Size = new System.Drawing.Size(90, 13);
            this.label1.TabIndex = 14;
            this.label1.Text = "Receiver address";
            // 
            // label2
            // 
            this.label2.AutoSize = true;
            this.label2.Location = new System.Drawing.Point(5, 191);
            this.label2.Name = "label2";
            this.label2.Size = new System.Drawing.Size(71, 13);
            this.label2.TabIndex = 15;
            this.label2.Text = "Receiver port";
            // 
            // Form1
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(340, 309);
            this.Controls.Add(this.label2);
            this.Controls.Add(this.label1);
            this.Controls.Add(this.textBoxComponentName);
            this.Controls.Add(this.textBoxReceiverPort);
            this.Controls.Add(this.textBoxReceiverAddress);
            this.Controls.Add(this.label4);
            this.Controls.Add(this.textBoxPosZ);
            this.Controls.Add(this.textBoxPosY);
            this.Controls.Add(this.textBoxPosX);
            this.Controls.Add(this.label3);
            this.Controls.Add(this.listBox1);
            this.Controls.Add(this.button1);
            this.Name = "Form1";
            this.Text = "UbitrackUbisenseSender";
            this.ResumeLayout(false);
            this.PerformLayout();

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

		private void locationClient_OnMove(string objectId, double x, double y, double z, 
			double yaw, double pitch, double roll, DateTime time)
		{
            if (m_socket == null)
                return;

			string person = (string)objectTable[(string)objectId];
            if ( person != m_selectedPerson )
                return;

            // create a time stamp
            DateTime unixStart = new DateTime(1970, 1, 1, 0, 0, 0);
            // System.Int64 timestamp = ( time.Ticks - unixStart.Ticks ) * 100; // what timezone?
            System.Int64 sendTime = (DateTime.Now.Ticks - unixStart.Ticks) * 100;

            // create and send packet
            System.Threading.Thread.CurrentThread.CurrentCulture = m_culture;
            string packet = String.Format("22 serialization::archive 3 {1} {0} 0 0 {3} 0 0 {4:f3} {5:f3} {6:f3} {2} 1 \n",
                m_componentName, m_componentName.Length, sendTime, sendTime, x, y, z );

            byte[] bytes = new byte[ packet.Length ];
            for ( int i = 0; i < packet.Length; i++ )
                bytes[ i ] = (byte)packet[ i ];

            m_socket.SendTo( bytes, m_endPoint );

            // update display
            object[] args = { x, y, z };
            this.Invoke(SetFormCoords, args);
		}

		private void button1_Click(object sender, System.EventArgs e)
		{
            try
            {
                // save selection
                if (listBox1.SelectedIndex < 0 )
                    throw new Exception( "Please select at least one person to log" );

                m_selectedPerson = (string)listBox1.SelectedItem;
                m_componentName = textBoxComponentName.Text.Trim();

                // create endpoint
                uint port = Convert.ToUInt16( textBoxReceiverPort.Text );
                IPHostEntry hostEntry = Dns.GetHostEntry(textBoxReceiverAddress.Text);
                m_endPoint = new IPEndPoint(hostEntry.AddressList[0], (int)port);

                // create socket
                m_socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
            }
            catch( Exception ex )
            {
			    MessageBox.Show( ex.Message, "Data Entry Error",
				    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
		}
	}
}
