/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */


/**
 * @ingroup driver_components
 * @file
 * Wiimote driver
 * This file contains the driver component to
 * get sensor and button data from a Nintendo Wiimote.
 * On Linux, it uses the libwiimote from the cwiid project.
 *
 * The received data is sent via a push interface.
 *
 * @author Florian Echtler <echtler@in.tum.de>
 */

#include <map>
#include <string>
#include <cstdlib>
#include <log4cpp/Category.hh>

#include <Ubitrack/Dataflow/PushSupplier.h>
#include <Ubitrack/Dataflow/ComponentFactory.h>
#include <Ubitrack/Dataflow/Component.h>
#include <Ubitrack/Measurement/Measurement.h>
#include <Ubitrack/Util/Exception.h>
#include <Ubitrack/Util/OS.h>

#include "cwiid.h"


namespace Ubitrack { namespace Drivers {

class WiimoteDriver
	: public Dataflow::Component
{

public:

	/** constructor */
	WiimoteDriver( const std::string& sName, boost::shared_ptr< Graph::UTQLSubgraph > pConfig );

	/** destructor */
	~WiimoteDriver();

	/** wiimote callback */
	void handle( int mesg_count, union cwiid_mesg mesg[] );

protected:

	Dataflow::PushSupplier< Measurement::Button > m_buttonOut;

	/** wiimote handle */
	cwiid_wiimote_t* wiimote;

};

} } // namespace Ubitrack::Drivers

