/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */


/**
 * @ingroup driver_components
 * @file
 * Xsens Raw Driver
 * This file contains the driver component to get raw acceleration,
 * magnetic field and rotation data from an Xsens MT9-B.
 *
 * The received data is sent via a push interface.
 *
 * @author Florian Echtler <echtler@in.tum.de>
 */

#include <map>
#include <string>
#include <cstdlib>
#include <log4cpp/Category.hh>

#include <Ubitrack/Dataflow/PushSupplier.h>
#include <Ubitrack/Dataflow/ComponentFactory.h>
#include <Ubitrack/Dataflow/Component.h>
#include <Ubitrack/Measurement/Measurement.h>
#include <Ubitrack/Measurement/TimestampSync.h>
#include <Ubitrack/Util/SerialPort.h>
#include <Ubitrack/Util/Exception.h>
#include <Ubitrack/Util/OS.h>


namespace Ubitrack { namespace Drivers {

class XsensRaw
	: public Dataflow::Component
{

public:

	/** constructor */
	XsensRaw( const std::string& sName, boost::shared_ptr< Graph::UTQLSubgraph > pConfig );

	/** destructor */
	~XsensRaw();

	/** thread method */
	void threadHandler();

protected:

	Dataflow::PushSupplier< Measurement::Vector3D > m_magOutput;
	Dataflow::PushSupplier< Measurement::Vector3D > m_accOutput;
	Dataflow::PushSupplier< Measurement::RotationVelocity > m_gyrOutput;

	boost::shared_ptr< boost::thread > m_thread;
	int m_keepRunning;

	Util::SerialPort* m_port;

};

} } // namespace Ubitrack::Drivers

