/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

/**
 * @ingroup driver_components
 * helper class to control Z800 HMD
 *
 * @author Florian Echtler <echtler@in.tum.de>
 */

#include "HMDControl.h"

#ifdef _WIN32

HMDControl::HMDControl() { } 
HMDControl::~HMDControl() { } 
void HMDControl::shdown( ) { }
void HMDControl::wakeup( ) { }
void HMDControl::mode3D( ) { }
int  HMDControl::getVer( ) { return 0; }

#else

#include <sys/ioctl.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>

#define IOCTL_GET_FIRMWARE_VERSION		1
#define IOCTL_SLEEP 					0x85
#define IOCTL_CYCLEBRIGHTNESS 			0x88
#define IOCTL_KEEPALIVE 				0x8A
#define IOCTL_SET_ENABLE_3D				0x8B

HMDControl::HMDControl()
{
	unsigned char buffer[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
	fd = open( "/dev/z800:0", O_RDWR );
	ioctl( fd, IOCTL_GET_FIRMWARE_VERSION, buffer, sizeof(buffer) );
	version = (buffer[0] << 8) + buffer[1];
	//printf( "Z800 Firmware Version: %x.%x\n", buffer[0], buffer[1] );
}

HMDControl::~HMDControl()
{
	shdown( );
	close( fd );
}

void HMDControl::shdown( ) { ioctl( fd, IOCTL_SLEEP            ); }
void HMDControl::wakeup( ) { ioctl( fd, IOCTL_KEEPALIVE        ); }
void HMDControl::mode3D( ) { ioctl( fd, IOCTL_SET_ENABLE_3D, 1 ); }
int  HMDControl::getVer( ) { return version; }

#endif

