/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

/**
 * @ingroup driver_components
 * helper class to read/decode Z800 rotation sensors
 *
 * @author Florian Echtler <echtler@in.tum.de>
 */

#include <Ubitrack/Measurement/Timestamp.h>
#include <Ubitrack/Math/RotationVelocity.h>
#include <Ubitrack/Math/Quaternion.h>
#include <Ubitrack/Math/Vector.h>

class RotationSensors {

	public:

		 RotationSensors();
		~RotationSensors();

		Ubitrack::Measurement::Timestamp receive( );

		Ubitrack::Math::RotationVelocity& getRotationSpeed();
		Ubitrack::Math::Quaternion& getMagneticRotation();
		Ubitrack::Math::Vector<3>& getAcceleration();

	private:

		void decode( unsigned char* _buf );

		int m_fd, m_count;

		int m_magRaw[3];
		int m_accRaw[3];
		int m_rotRaw[6];

		int m_magOffs[3];
		int m_rotOffs[6];

		Ubitrack::Math::RotationVelocity m_rotSpeed;
		Ubitrack::Math::Quaternion       m_magRot;
		Ubitrack::Math::Vector<3>        m_accel;
};

