#ifndef _CMMSPECIFIC_H
#define _CMMSPECIFIC_H
//////////////////////////////////////////////////////////////////////////////
//  File:      CmmSpecific.h
//
//  Summary:   Contains definition of structures that are passed as
//			   parameters to the CmmSpecific() method.
//---------------------------------------------------------------------
//  This file is part of the USB FaroArm SDK Code Samples.
//
//  Copyright (C) 2002 FARO Technologies, Inc.  All rights reserved.
//
//  FARO Technologies, Inc. grants you a non-exclusive, royalty-free
//  license to copy, modify, and distribute the software solely for
//  the purpose of facilitating interoperability between your software
//  and the FARO Technologies portable measurement arms.
//
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE
// AND NON-INFRINGEMENT.  WE DISCLAIM ALL LIABILITY FOR ANY LOSS OR DAMAGE
// SUFFERED BY YOUR OR ANY THIRD PARTY AS A RESULT OF USING OR DISTRIBUTING
// THIS SOFTWARE, INCLUDING DIRECT, INDIRECT, INCIDENTAL, SPECIAL OR
// CONSEQUENTIAL DAMAGES (INCLUDING LOST SAVINGS, LOST PROFIT OR ATTORNEY
// FEES), HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, EVEN IF
// FARO HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//////////////////////////////////////////////////////////////////////////////

#define BIT00 0x00000001
#define BIT01 0x00000002
#define BIT02 0x00000004
#define BIT03 0x00000008
#define BIT04 0x00000010

#define CMM_UPDATE_DETECT			BIT00        
#define CMM_TIMEOUT_DETECT			BIT01
#define CMM_ERROR_DETECT			BIT02
#define CMM_MOUSE_DETECT			BIT03
#define CMM_INACCURATE_DATA_DETECT	BIT04

#define DLL_OPEN_CMM				1
#define DLL_CLOSE_CMM				2
#define DLL_ARM_INFO				3
#define DLL_PROBE_DIAM				4
#define DLL_POS_UPDATE				5
#define DLL_END_UPDATE				6
#define DLL_DIALOG_CMM				7
//#define DLL_						8 //Obsolete
#define DLL_PROBE_DIALOG			9
#define DLL_CALIBRATE_ENTER			10
#define DLL_CALIBRATE_HOLE			11
#define DLL_CALIBRATE_SPHERE		12
//#define DLL_						13 // obsolete
#define DLL_MOUSE_MODE_ON			14
#define DLL_MOUSE_MODE_OFF			15
#define DLL_KEY_MODE_ON				16
#define DLL_KEY_MODE_OFF			17
#define DLL_SETTINGS_DIALOG			18
#define DLL_TEMPERATURE_DIALOG		19
#define DLL_ANGLES_DIALOG			20
#define DLL_TERMINAL_DIALOG			21
#define DLL_HELP_DIALOG				22
#define DLL_UNITS_IN				23
#define DLL_UNITS_MM				24
//#define DLL_						25  //Obsolete
// #define DLL_               26
#define DLL_START_MEASUREMENT		27
#define DLL_STOP_MEASUREMENT		28

#define DLL_GET_PROBE_INFO			38
#define DLL_SET_PROBE_INFO			39
#define DLL_INITIALIZE				53  // initialize path of configuration file
#define DLL_START_MEASUREMENT_CMD	54
#define DLL_STOP_MEASUREMENT_CMD	55
#define DLL_GET_DEVICE_SETUP		56	// what model, adm?
#define DLL_CALIBRATE_PLANE			1011	// plane calibation

#define GOOD_KEY					0xc1c2c3c4

#define MAX_POINTS_FOR_A_SCAN		1024

struct vArmInfo
{
	int		nFuncType;	// = DLL_ARM_INFO
	char	sArmData[32];
	char	sCertificationDate[16];
	char	sCalibrationDate[16];
	int		nSerindVersion;
	float	fUnits;
};

struct vOpenInfo
{
	int		nFuncType;	// = DLL_OPEN_CMM or DLL_INITIALIZE
	HWND	hWnd;
	WCHAR	sSaveDir[128];
	void	(*HelpCallBack)();
};

struct vGetProbeInfo
{
	int		nFuncType;	// = DLL_GET_PROBE_INFO
	double	dProbeDiameter;
	double	dAdapterHeight;
	double	dEndEffectorError;
	WCHAR	sProbeName[128];
	WCHAR	sAdapterName[128];
};

struct vProbeDiam
{
	int		nFuncType;	// = DLL_PROBE_DIAM
	double	dEndEffectorXYZ[4][3];	
	double	dEndEffectorDiameter[4];
	double	dEndEffectorError;
	int		nCurrentEndEffector;
};

struct vSetProbeInfo
{
	int		nFuncType;	// = DLL_SET_PROBE_INFO
	double	dProbeDiameter;
	double	dAdapterHeight;
	WCHAR	sProbeName[128];
	WCHAR	sAdapterName[128];
};

struct vProbeDlgInfo
{
	int		nFuncType;	// = DLL_PROBE_DIALOG
	WCHAR	sProbeFileName[300];
	struct vSetProbeInfo setProbe;
};

struct vKeyMode
{
	int		nFuncType;	// = DLL_KEY_MODE_ON
	int		nFrontButton;
	int		nBackButton;
};

struct vChangeProbe
{
	int		nFuncType;	// = DLL_SET_PROBE;
	int		nProbe;
	double	dDiameter;
};

struct PositionUpdate
{
	int 	nFuncType;	// = DLL_POS_UPDATE
	char    cUpdateFlag;
	unsigned uFnumber;
	char	cFrontButton;
	char	cBackButton;
	double	dPosition[8];
	HANDLE	hPositionUpdateEvent;
	UINT	unKey;
	int		nPnts;
	double	dPoints[MAX_POINTS_FOR_A_SCAN][3];
};

#endif //_CMMSPECIFIC_H
