import java.math.BigInteger;
import ubitrack.*;


public class console {
    static {
	System.loadLibrary("ubitrack_java");
    }

    public static void main(String argv[]) {
	if (argv.length < 1) {
	    System.out.println("Usage: java console <utql_file> <appsink_pattern_id>");
	    System.out.flush();
	    System.exit(1);
	}

	// Initialize logging in UbiTrack library
	ubitrack.initLogging();

	System.out.println("Creating instance of SimpleFacade...");
	System.out.flush();
	SimpleFacade f = new SimpleFacade();
	if ( f.getLastError() != null ) {
	    System.out.println("Problem in constructor");
	    System.out.flush();
	    return;
	}

	System.out.println("Instantiate our SimpleDataflowObserver derivate");
	System.out.flush();
	SimpleDataflowObserver dtflObs = new MyDataflowObserver();

	System.out.println("Register our SimpleDataflowObserver derivate");
	System.out.flush();
	f.setDataflowObserver(dtflObs);

	System.out.println("Instantiating network from " + argv[0]);
	System.out.flush();
	if ( ! f.loadDataflow( argv[0] ) ) {
	    System.out.println("Problem in loadDataflow()");
	    System.out.flush();
	    System.exit(1);
	}

 	System.out.println("Instantiate our SimplePoseReceiver derivate");
 	System.out.flush();
 	SimplePoseReceiver pr = new MyPoseReceiver();

	if (argv.length == 2) {
		System.out.println("Register our SimplePoseReceiver derivate");
		System.out.flush();
		if ( ! f.setPoseCallback(argv[1], pr) ) {
			System.out.println("Problem in setPoseCallback()");
			System.out.flush();
			System.exit(1);
		}
	}

	System.out.println("Starting dataflow");
	System.out.flush();
	f.startDataflow();

	System.out.println("Press RETURN to stop...");
	System.out.flush();
	try {
	    System.in.read();
	} catch (Exception e) {
	    System.out.println("Error occurred reading from standard input.");
	    System.out.flush();
	}

	f.stopDataflow();
	System.out.println("Done");
	System.out.flush();
	System.exit(0);
    }
}


class MyPoseReceiver extends SimplePoseReceiver {
    public void receivePose(SimplePose pose) {
	// Retrieve pose data 
	double [] trans = new double[3];
	double [] rot = new double[4];
	trans[0] = pose.getTx();
	trans[1] = pose.getTy();
	trans[2] = pose.getTz();
	rot[0] = pose.getRx();
	rot[1] = pose.getRy();
	rot[2] = pose.getRz();
	rot[3] = pose.getRw();
	// Retrieve timestamp
	BigInteger ts = pose.getTimestamp();

	System.out.println("Received pose event, translation: " 
			   + trans[0] + " " + trans[1] + " " + trans[2] 
			   + ", rotation: "
			   + rot[0] + " " + rot[1] + " " + rot[2] + " " + rot[3]
			   + ", timestamp: "
			   + ts.toString());
    }
}


class MyDataflowObserver extends SimpleDataflowObserver {
  public void notifyAddComponent(String sPatternName, String sComponentName) {
      System.out.println("Component \"" + sComponentName + "\" (corresponding to UTQL pattern \"" + sPatternName + "\") added to dataflow network");
  }

  public void notifyDeleteComponent(String sPatternName, String sComponentName) {
      System.out.println("Component \"" + sComponentName + "\" (corresponding to UTQL pattern \"" + sPatternName + "\") removed from dataflow network");
  }
}
