#!/bin/bash


# Set classpath, it also has to contain the ubitrack wrapper jar

export CLASSPATH=".:../../../lib/ubitrack.jar"


# Set library search path. It has to contain at least the lib/ sub-directory
# (contains the native wrapper library and the Ubitrack library itself along
# with its plugins) because otherwisee, the Java VM cannot find the native
# wrapper (see also System.getProperty( "java.library.path" )). Furthermore, it
# may contain all non-standard libraries needed by Ubitrack. Standard libraries
# are always searched for by the linker in /lib/ or /usr/lib. Ubitrack comes
# with all needed libraries except the standard libraries and maybe boost
# because boost may not be available as a package on your platform, at least not
# in a recent version. Thus, you probably have to specify to the linker the lib/
# subdirectory of your Ubitrack installation as well as the installation
# directory of boost. The prefered way to specify library paths to the linker is
# adding them to /etc/ld.so.conf and running ldconfig as root. This is the
# preferred method since guarantees the best launch performance. On the other
# hand, it is annoying to run ldconfig whenever new plugins for Ubitrack arise
# during development. In this situation (or if your account does not have
# permission to edit ld.so.conf and run ldconfig), adding them to the
# environment variable LD_LIBRARY_PATH is the better option.

# The following setting assumes that all libraries needed by Ubitrack (including
# boost) can be found via the prefered mechanism and only the Ubitrack stuff has
# to be additionally specified.

export LD_LIBRARY_PATH="../../../lib"


# (Re-)compile console application

javac console.java


# Invoke console application with sample dataflow configuration (works without
# any hardware) and register an application push sink for pattern with ID
# TestReceiver

java console ../../../doc/utql/examples/ExamplePushSink.xml TestReceiver


# Invoke console application with snowman marker tracker demo configuration 

#java console ../../../doc/utql/MarkerTrackerDemoResponse.xml
