/*
 * Ubitrack - Library for Ubiquitous Tracking
 * Copyright 2006, Technische Universitaet Muenchen, and individual
 * contributors as indicated by the @authors tag. See the 
 * copyright.txt in the distribution for a full listing of individual
 * contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */


#include <stdlib.h>
#include <iostream>

#include <Ubitrack/Util/Logging.h>
#include <Ubitrack/Util/Exception.h>
#include <Ubitrack/Facade/SimpleFacade.h>
#include <Ubitrack/Facade/SimpleDatatypes.h>

#include <log4cpp/PropertyConfigurator.hh>


using namespace Ubitrack;


class MyPoseReceiver : public Ubitrack::Facade::SimplePoseReceiver 
{
public:
    MyPoseReceiver();
    virtual void receivePose( const Ubitrack::Facade::SimplePose& pose ) throw();
};


MyPoseReceiver::MyPoseReceiver() {}


void MyPoseReceiver::receivePose( const Ubitrack::Facade::SimplePose& pose ) throw() 
{
    std::cout << "Received Pose event, translation: " 
	      << pose.tx << " " << pose.tx << " " << pose.tz 
	      << ", rotation: " 
	      << pose.rx << " " << pose.ry << " " << pose.rz << " " << pose.rw 
	      << std::endl << std::flush;
}



int main( int ac, char** av )
{
    try
    {
	if (ac < 2)
	{
	    std::cerr << "Usage: " << av[0] << " filename" << std::endl;
	    exit (1);
	}

	try {
	    log4cpp::PropertyConfigurator::configure( "log4cpp.conf" );
	} 
	catch ( ... ) { }
	Ubitrack::Util::initLogging();

	std::cout << "Loading components..." << std::endl << std::flush;
	Ubitrack::Facade::SimpleFacade f;

	std::cout << "Instantiating network from" << av[1] << "..." << std::endl << std::flush;
	if ( !f.loadDataflow( av[1] ) )
	    UBITRACK_THROW( "Problem in loadDataflow()" );

	std::cout << "Instantiating our derivate of SimplePoseReceiver" << std::endl << std::flush;
	Ubitrack::Facade::SimplePoseReceiver * pr = new MyPoseReceiver();
	std::cout << "Registering our derivate of SimplePoseReceiver" << std::endl << std::flush;
	if ( ! 	f.setPoseCallback( "appsink", pr ) )
	    UBITRACK_THROW( "Problem in setPoseCallback()" );

	std::cout << "Starting dataflow" << std::endl;
	f.startDataflow();

	std::cout << "Press RETURN to stop..." << std::endl << std::flush;
	std::cin.get();

	f.stopDataflow();
	std::cout << "Finished, cleaning up..." << std::endl << std::flush;
    }
    catch( Util::Exception& e )
    {
	std::cerr << e << std::endl;
    }

}
