#include <boost/test/unit_test.hpp>
#include <Ubitrack/Dataflow/ComponentFactory.h>
#include <Ubitrack/Dataflow/Port.h>
#include <Ubitrack/Util/Exception.h>

using namespace Ubitrack::Dataflow;

void TestDynamicLoad()
{
	std::vector< std::string > libs;

#ifdef _WIN32
	libs.push_back( "TestComponent.dll" );
#else
	libs.push_back( "libTestComponent.so" );
#endif

	// load some libraries
	boost::shared_ptr< ComponentFactory > pCf;
	pCf.reset( new ComponentFactory( libs ) );

	{
		// try to instatiate a component
		boost::shared_ptr< Component > pComponent = pCf->createComponent( "TestComponent", "myTestComp", boost::shared_ptr< Ubitrack::Graph::UTQLSubgraph > ( new Ubitrack::Graph::UTQLSubgraph() ) );

		// check if it has required ports
		BOOST_CHECK_NO_THROW( pComponent->getPortByName( "MagicPort" ) );
		BOOST_CHECK_THROW( pComponent->getPortByName( "SomeOtherPort" ), Ubitrack::Util::Exception );
	}
	// try to instantiate a non-exisiting component
	BOOST_CHECK_THROW( pCf->createComponent( "NotExistingComponent", "lakdsjf", boost::shared_ptr< Ubitrack::Graph::UTQLSubgraph > ( new Ubitrack::Graph::UTQLSubgraph() ) ), std::runtime_error );
}
