#include <boost/test/unit_test.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/bind.hpp>

#include <Ubitrack/Dataflow/PullConsumer.h>
#include <Ubitrack/Dataflow/PullSupplier.h>
#include <Ubitrack/Dataflow/Component.h>


// start an anonymous namespace for test-internal classes
namespace {

using namespace Ubitrack::Dataflow;

template< class EventType > class PullTest
	: public Component
{
public:
	PullTest()
		: Component( "PullTest" )
		, m_Supplier( "Out", *this, boost::bind( &PullTest< EventType >::generate, this, _1 ) )
		, m_Consumer( "In", *this )
	{
		m_Consumer.connect( m_Supplier );
		m_Supplier.connect( m_Consumer );
	}

	~PullTest()
	{
		m_Consumer.disconnect( m_Supplier );
		m_Supplier.disconnect( m_Consumer );
	}

	void test( const EventType& data )
	{
		m_pData = &data;
		BOOST_CHECK_EQUAL( m_Consumer.get( 0 ), data );
	}

protected:
	EventType generate( Ubitrack::Measurement::Timestamp )
	{
		return EventType( *m_pData );
	}

	PullSupplier< EventType > m_Supplier;
	PullConsumer< EventType > m_Consumer;

	const EventType* m_pData;
};

} // anonymous namespace


void TestPull()
{
	// test pushing ints
	boost::scoped_ptr< PullTest< int > > testInt( new PullTest< int > );
	testInt->test( 4 );
	testInt->test( -263 );
	testInt->test( 0 );

	// test pushing strings
	boost::scoped_ptr< PullTest< std::string > > testString( new PullTest< std::string > );
	testString->test( std::string() );
	testString->test( "Hi" );
	testString->test( "adfasfd" );
}
